// Control.cpp  Copyright 1998 Steve Ludtke (4/98)
// This is the core object for the 'ctfit' package
// It primarily handles user interface operations. Most of the 
// real work is handed off to CtfData

#include "Control.h"
#include <qfiledlg.h>
#include <qapp.h>
#include <qfont.h>
#include <qcolor.h>
#include <qmsgbox.h> 
#include <qprogressdialog.h>
#include <qmessagebox.h>
#include <math.h>
#include "OneLiner.h"
#include "ColorListItem.h"

#define Inherited ControlData

extern QString dfdir;

QString simpdate(const QDate &d) {
	QString ret;
	if (!d.isValid()) return("MM/DD/YY");
	ret=QString::number(d.month())+"/"+QString::number(d.day())+"/"+
		QString::number(d.year()>1999?d.year()-2000:d.year()-1900);
	return ret;
}

QDate simpdate(const QString &dt) {
int y,m,d;
QDate ret=QDate::currentDate();

if (sscanf((const char *)dt,"%d/%d/%d",&m,&d,&y)!=3) return ret;
if (y<80) y+=2000;
if (y<100) y+=1900;
ret.setYMD(y,m,d);

return ret;
}


Control::Control(QWidget* parent,const char* name):Inherited( parent, name ) {
QPopupMenu *m1,*m2;

//initMetaObject();

parmDisp=NULL;
eqed=NULL;

plotwin=new PlotCtf(NULL,"CTF Plot");
plotwin->setData(datalist);
plotwin->show();
plotwin->newplots();

imgwin=new Image;
imgwin->resize(300,300);
imgwin->show();

rtwin=NULL;

curmemfile="ctfparm.txt";

ImShape *sh = new ImShape;
sh->type=ImShape::CENELLIPSE;
sh->color=red;
imgwin->addShape(sh);

// This restores the window geometries
FILE *in = in=DFopenr("ctfitwin");
if (in) {
	int x,y,w,h;

#ifndef SGI
	if (fscanf(in," %d %d %d %d",&x,&y,&w,&h)==4) {
		if (x<0) x=0;
		if (y<0) y=0;
		move(x,y);
	}
	if (fscanf(in," %d %d %d %d",&x,&y,&w,&h)==4) {
		if (x<0) x=0;
		if (y<0) y=0;
//		plotwin->setGeometry(x,y,w,h);
		plotwin->move(x,y);
	}
	if (fscanf(in," %d %d %d %d",&x,&y,&w,&h)==4) {
		if (x<0) x=0;
		if (y<0) y=0;
		imgwin->setGeometry(x,y,w,h);
	}
#endif
	fclose(in);
}

mem.setAutoDelete(TRUE);

// FILE MENU
m1=new QPopupMenu(NULL,"File");
menu->insertItem("File",m1,100);
//m1->insertItem("Open Whole Micrograph",this,SLOT(Mopenmicro()),ALT+Key_O);
//m1->insertItem("Split focal pair",this,SLOT(Msplitmicro()),0);
m1->insertItem("Open Single Image(s)",this,SLOT(Mopen1clip()),0);
m1->insertItem("Open Particle Set",this,SLOT(Mopenclips()),0);
m1->insertItem("Split Particle Set",this,SLOT(MopenclipsS()),0);
m1->insertItem("Split Set by CTF",this,SLOT(MopenclipsC()),0);
m1->insertItem("Open Set (oversample)",this,SLOT(Mopenclipsos()),0);
m1->insertItem("Open Real FFT",this,SLOT(MopenRealFFT()),0);
m1->insertItem("Struc. Fac. -> Image",this,SLOT(Mopenstruc()),0);
m1->insertItem("Save FFT to MRC",this,SLOT(Msavefft()));
m1->insertSeparator();
m1->insertItem("Print",this,SLOT(Mprint()),ALT+Key_P);
m1->insertSeparator();
m1->insertItem("Exit",this,SLOT(Mquit()),ALT+Key_X);

// PROCESS MENU
m1=new QPopupMenu(NULL,"Process");
menu->insertItem("Process",m1);
m1->insertItem("Phase Correct",this,SLOT(Mfixset()),0);
m1->insertItem("Set CTF Parms",this,SLOT(Msetparm()),0);
m1->insertItem("Focal Pair Merge",this,SLOT(Mpair()),0);
m2=new QPopupMenu(NULL,"Fit");
m1->insertItem("Fit",m2);
m2->insertItem("Generic Fit",this,SLOT(doFitQ()),0);
m2->insertItem("Fit with x-ray",this,SLOT(doFitX()),0);
m2=new QPopupMenu(NULL,"Simulate");
m1->insertItem("Simulate",m2);
m2->insertItem("RT CTF Sim",this,SLOT(MrtCTF()),0);
m2->insertItem("Apply CTF 2D",this,SLOT(Mapply()),0);
m2->insertItem("Apply CTF 3D",this,SLOT(Mapply3d()),0);
m2->insertItem("Apply CTF 3D w noise",this,SLOT(Mapply3dn()),0);
m2->insertItem("Simple CTF Cor 2D/3D",this,SLOT(Munapply()),0);
m2->insertItem("Add noise 2D",this,SLOT(Mnoise()),0);
m2->insertItem("Set Struc Fac. 3D",this,SLOT(MsetSF()),0);
m2=new QPopupMenu(NULL,"Advanced");
m1->insertItem("Advanced",m2);
m2->insertItem("Edit User Fn",this,SLOT(Mseteqn()),0);
m2->insertItem("Edit all equations",this,SLOT(MshowEq()));
m2->insertItem("Wiener Filter particles",this,SLOT(MWFilter()),0);
m2->insertItem("Wiener Filter image",this,SLOT(MWFilterMRC()),0);
m2->insertItem("Save Plot as Spreadsheet",this,SLOT(Msavedata()));
m2->insertItem("Save 1 column",this,SLOT(Msavedata1c()));
m2->insertItem("Save Average Profile",this,SLOT(Msaveavg()),0);
m2->insertItem("Show parameters",this,SLOT(showParm()));
menu->insertSeparator();

// HELP MENU
m1=new QPopupMenu(NULL,"Help");
menu->insertItem("Help",m1);
m1->insertItem("Help",this,SLOT(Mhelp()),0);
m1->insertItem("About CTFIT",this,SLOT(Mabout()),0);

df->setMin(.01);
df->setMax(5.0);
envel->setMax(20.0);
amp->setMax(5.0);
ampc->setMax(100.0);
expa->setMax(10.0);
expw->setMax(80.0);
gaussa->setMax(10.0);
gaussw->setMax(25.0);
stigang->setMin(0);
stigang->setMax(360.0);
stigdf->setMin(-3.0);
stigdf->setMax(0);

//microscope->insertItem("");
//specimen->insertItem("");
//scanner->insertItem("");

#ifdef CHIU
microscope->insertItem("Default");
microscope->insertItem("JEOL 1200 (chiu)");
microscope->insertItem("JEOL 4000 (chiu)");
microscope->insertItem("JEOL 3000 (chiu)");
#else
microscope->insertItem("Default");
#endif

initMem();
envbflag=0;

QObject::connect(df,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(envel,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(amp,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(ampc,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(exps4,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(expa,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(expw,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(gaussa,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(gaussw,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(stigang,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(stigdf,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
QObject::connect(cfilmsf,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));
//QObject::connect(ptclsize,SIGNAL(newValue(float)),this,SLOT(parmchange(float)));

QObject::connect(sfnone,SIGNAL(clicked()),this,SLOT(setsfnone()));
QObject::connect(sfsph,SIGNAL(clicked()),this,SLOT(setsfsph()));
QObject::connect(sffile,SIGNAL(clicked()),this,SLOT(setsffile()));

QObject::connect(voltage,SIGNAL(lostFocus()),this,SLOT(vchange()));
QObject::connect(Cs,SIGNAL(lostFocus()),this,SLOT(vchange()));
QObject::connect(apix,SIGNAL(lostFocus()),this,SLOT(apixchange()));
QObject::connect(voltage,SIGNAL(returnPressed()),this,SLOT(vchange()));
QObject::connect(Cs,SIGNAL(returnPressed()),this,SLOT(vchange()));
QObject::connect(apix,SIGNAL(returnPressed()),this,SLOT(apixchange()));
QObject::connect(newapix,SIGNAL(clicked()),this,SLOT(magchange()));

QObject::connect(ebtog,SIGNAL(clicked()),this,SLOT(ebTog()));

QObject::connect(datanewsim,SIGNAL(clicked()),this,SLOT(dataNewsim()));
QObject::connect(datadel,SIGNAL(clicked()),this,SLOT(dataDel()));
QObject::connect(datalist,SIGNAL(highlighted(int)),this,SLOT(dataHi(int)));
QObject::connect(datalist,SIGNAL(selected(int)),this,SLOT(dataSel(int)));
QObject::connect(dataname,SIGNAL(returnPressed()),this,SLOT(newDataName()));

QObject::connect(Memset,SIGNAL(clicked()),this,SLOT(memSet()));
QObject::connect(memdel,SIGNAL(clicked()),this,SLOT(memDel()));
QObject::connect(memrcl,SIGNAL(clicked()),this,SLOT(memRcl()));
QObject::connect(memnew,SIGNAL(clicked()),this,SLOT(memNew()));
QObject::connect(memlist,SIGNAL(highlighted(int)),this,SLOT(memHi(int)));
QObject::connect(memlist,SIGNAL(selected(int)),this,SLOT(memRcl(int)));
QObject::connect(memname,SIGNAL(returnPressed()),this,SLOT(newMemName()));
QObject::connect(memfile,SIGNAL(clicked()),this,SLOT(setmemFile()));

//QObject::connect(dofit,SIGNAL(clicked()),this,SLOT(doFit()));
//QObject::connect(dofitq,SIGNAL(clicked()),this,SLOT(doFitQ()));

QObject::connect(imgwin,SIGNAL(mouseMove(QMouseEvent *)),this,SLOT(newS(QMouseEvent *)));
QObject::connect(microscope,SIGNAL(activated(const QString &)),this,SLOT(newMicroscope(const QString &)));

updateDList();

//printf("Df font: %s %d\n",QFont::defaultFont().family(),QFont::defaultFont().pointSize());
}

Control::~Control() {

//delete plotwin;
delete imgwin;
saveMem();
//printf("Memory saved\n");
}

void Control::showParm() {
	if (!parmDisp) {
		parmDisp=new QLineEdit(NULL);
		parmDisp->resize(300,25);
	}
	parmDisp->show();
}

void Control::newMicroscope(const QString &t) {
QString s;

s=t.stripWhiteSpace();
if (s.isEmpty() || s.isNull() || s=="(null)") return;

#ifdef CHIU
QString v,cs;
if (t=="JEOL 1200 (chiu)") { v="100"; cs="5.6"; }  
else if (t=="JEOL 4000 (chiu)") { v="400"; cs="4.1"; }
else if (t=="JEOL 3000 (chiu)") { v="300"; cs="1.6"; }
if (!v.isEmpty()) { 
	voltage->setText(v);
	Cs->setText(cs);
}
#endif

for (int i=0; i<microscope->count(); i++) {
	if (microscope->text(i)==s) { 
		microscope->setCurrentItem(i); 
		if (current()) current()->microscope=s;
		return; 
	}
}
microscope->insertItem(s);
microscope->setCurrentItem(microscope->count()-1);
if (current()) current()->microscope=s;
}

void Control::ebTog() {

if (envbflag) {
	envbflag=0;
	ebtog->setText("Envelope:");
	envel->setMax(20.0);
	envel->setValue(sqrt(envel->Value())*2.0/PI);
	return;
}
else {
	envbflag=1;
	ebtog->setText("B:");
	envel->setMax(1000.0);
	envel->setValue(SQR(envel->Value()*PI/2.0));
	return;
}

}

void Control::updateDList() {
int i,j;
char s[80];

//printf("Update\n");
datalist->viewport()->update();
//plotwin->setCurrent(current());
//plotwin->newplots();
dataHi(-1);

}


void Control::Mopenmicro() { 
warn("Not implimented yet!");
return;
/*QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;
QFile file(s);

EMData *d1,*d2;
CtfData *cd;

QApplication::setOverrideCursor( waitCursor );

d1=new EMData;
d1->readImage(s.data(),0);

if (d1->isComplex()) { d2=d1; d1=d2->doIFT(); d2->gimmeFFT(); }
else { d1->normalize(); d2=d1->doFFT(); d1->gimmeFFT(); }

cd=new CtfData(d1->ySize()>400?400:d1->ySize());
data->append(cd);

cd->copy(current);
current=cd;

cd->setRealImage(d1);
cd->setFFTImage(d2);

cd->setText(s.data());

updateDList();

QApplication::restoreOverrideCursor();
*/
}

void Control::Msplitmicro() { 
EMData *d1,*d2,*c1,*c2,*c3;
int x,y,w,h,dx,dy;

warn("Not implimented yet!");
return;
QString s1( QFileDialog::getOpenFileName() );
if ( s1.isNull() ) return;
QString s2( QFileDialog::getOpenFileName() );
if ( s2.isNull() ) return;

d1=new EMData;
d2=new EMData;
if (d1->readImage(s1,0)) { printf("Image 1 read failed\n"); return; }
if (d2->readImage(s2,0)) { printf("Image 2 read failed\n"); return; }

printf("Image 1 is %dx%d\n",d1->xSize(),d1->ySize());
printf("Image 2 is %dx%d\n",d2->xSize(),d2->ySize());

w=d1->xSize()<d2->xSize()?d1->xSize():d2->xSize();
h=d1->ySize()<d2->ySize()?d1->ySize():d2->ySize();

c1=d1->clip(w/2-512,h/2-512,1024,1024);
c2=d2->clip(w/2-512,h/2-512,1024,1024);

c3=c1->RTFAlign(c2,NULL);

c1->writeMRC("x1.mrc");
c2->writeMRC("x2.mrc");
c3->writeMRC("x3.mrc");

dx=(int)c3->Dx();
dy=(int)c3->Dy();

delete d1;
delete d2;
}

void Control::Mopen1clip() { 
Mopen1clip(NULL);
}

void Control::Mopenclips() {
Mopenclips(NULL);
}

// This will read in a set of single image files
void Control::Mopen1clip(const char *file) { 
EMData *d1,*d2,*d3;
CtfData *cd;
int i,n,nx,ny;
char *ss,sff[81];

QStringList s;
if (file) s.append(file);
else {
	s=QFileDialog::getOpenFileNames(NULL,dfdir);
}
n=s.count();
//printf("%d\n",n);
//for (i=0; i<n; i++) printf("%s\n",(const char *)s[i]);
if ( n==0 ) return;

QApplication::setOverrideCursor( waitCursor );


QProgressDialog progress( "Reading Clips", "Abort", n, this, 0 ,TRUE );
progress.setProgress( 0 );

d1=new EMData;
d1->readImage(s[0],0);
nx=d1->xSize();
ny=d1->ySize();

sff[0]=0;
mrcH *mrch = d1->getMRCHed();
if (mrch) {
	mrch->labels[1][79]=0;
	strcpy(sff,mrch->labels[1]);
}
if (access(sff,R_OK)) sff[0]=0;

if (nx!=ny && !d1->isComplex() && (nx<1024 || ny<1024)) {
	printf("'%s' is not square and is less than 1024 pixels. %d != %d\n",(const char *)s[0],nx,ny);
	return;
}


ss=(char *)malloc(strlen(d1->Name())+1);
strcpy(ss,d1->Name());

// if input is complex, we can only handle 1 file
if (d1->isComplex()) {
	d3=d1;
	d1=NULL;
	cd=new CtfData(d3->ySize()>250?250:d3->ySize());
	datalist->insertItem(cd);
	cd->copy(current());
	datalist->setCurrentItem(cd);
	newData();

	if (ss[0]=='!'&&ss[1]=='-') cd->setParmStr(ss);
	free(ss);

	cd->setRealImage(NULL);
	cd->setFFTImage(d3);

	cd->setLName(s[0]);
	cd->setFile(s[0]);
	if (sff[0]) cd->setStrucMode(-1,sff);

	updateDList();

	parmchange(0);
	QApplication::restoreOverrideCursor();
	return;
}

if (nx!=ny) {
	printf("Large rectangular image, clipping 1k x 1k area from center\n");
	d3=d1->clip((nx-1024)/2,(ny-1024)/2,1024,1024);
	delete d1;
	d1=d3;
}
d1->applyMask(-1,6);
d1->normalize();
d3=d1->doFFT();
d1->gimmeFFT();
d3->multConst(d3->ySize());
d3->ri2ap();

for (i=1; i<n; i++) {
	d1->readImage(s[i],0);
	if (d1->xSize()!=nx || d1->ySize()!=ny) {
		printf("'%s' is the wrong size %d != %d || %d != %d\n",(const char *)s[i],d1->xSize(),nx,d1->ySize(),ny);
		continue;
	}
	d1->applyMask(-1,6);
	d1->normalize();
	d2=d1->doFFT();
	d1->gimmeFFT();
	d2->multConst(d2->ySize());
	d3->addIncoherent(d2);
	delete d2;
	progress.setProgress( i );
	if ( progress.wasCancelled() ) n=i;
}

delete d1;
d3->multConst(1.0/sqrt((float)n));
//d3->normalize();
//d3->ri2ap();
//d3->writeMRC("xxx.img");

cd=new CtfData(d3->ySize()>250?250:d3->ySize());
datalist->insertItem(cd);
cd->copy(current());


cd->setRealImage(NULL);
cd->setFFTImage(d3);

cd->setLName(s[0]);
cd->setFile(s[0]);

if (ss[0]=='!'&&ss[1]=='-') cd->setParmStr(ss);
free(ss);

if (sff[0]) cd->setStrucMode(-1,sff);

datalist->setCurrentItem(cd);
newData();
updateDList();
parmchange(0);

QApplication::restoreOverrideCursor();

}

void Control::MopenRealFFT() {
QString s( QFileDialog::getOpenFileName(dfdir) );
if ( s.isNull() ) return;
QFile file(s);

EMData *d1,*d2;
CtfData *cd;
float *da1,*da2;
int i;

QApplication::setOverrideCursor( waitCursor );

d2=new EMData;
d2->readImage(s,0);

d1=new EMData;
d1->setComplex(1);
d1->setSize(d2->xSize()*2,d2->ySize(),1);

da1=d1->getData();
da2=d2->getDataRO();
for (i=0; i<d2->xSize()*d2->ySize(); i++) da1[i*2]=da2[i];
d1->doneData();
d2->doneData();

delete d2;

if (d1->isComplex()) { d2=d1; d1=d2->doIFT(); d2->gimmeFFT(); }
else { d1->normalize(); d2=d1->doFFT(); d1->gimmeFFT(); d2->multConst(d2->ySize()); }

d2->ri2ap();

cd=new CtfData(d1->ySize()>250?250:d1->ySize());
datalist->insertItem(cd);
cd->copy(current());
datalist->setCurrentItem(cd);
newData();

cd->setRealImage(d1);
cd->setFFTImage(d2);

cd->setLName(s);
cd->setFile(s);

updateDList();

QApplication::restoreOverrideCursor();

}

void Control::Mopenstruc() { 
QString s( QFileDialog::getOpenFileName(dfdir) );
if ( s.isNull() ) return;
QFile file(s);

EMData *d1,*d2;
CtfData *cd;
FILE *in;
char t[121];
int i,n=0,r0;
float *x,*y,ds;

QApplication::setOverrideCursor( waitCursor );

in=fopen(s,"r");
while (fgets(t,120,in)!=NULL) {
	if (sscanf(t," %f %f",&ds,&ds)==2) n++;
}
printf("Radial function has %d elements\n",n);
if (n==0) { warn("No data read\n"); return; }

rewind(in);
x=(float*)malloc(n*sizeof(float));
y=(float*)malloc(n*sizeof(float));
for (i=0; i<n; i++) {
	fgets(t,120,in);
	if (sscanf(t," %f %f",x+i,y+i)!=2) i--;
}
fclose (in);

ds=x[1]-x[0];
if (ds!=x[2]-x[1]) warn("Unevenly sampled function\nResult Invalid!");
r0=(int)floor(x[0]/ds+.5);
for (i=0; i<n; i++) x[i]=r0+i;

printf("Image size %d\n",2*(r0+n));

d1=new EMData;
d1->setSize((r0+n)*2+2,(r0+n)*2,1);
d1->setComplex(1);
d1->one();

d1->applyRadFn(n,r0,1,y,1);

free(x);
free(y);

d1->ri2ap();

cd=new CtfData((r0+n)>250?250:(r0+n));
datalist->insertItem(cd);
cd->copy(current());
datalist->setCurrentItem(cd);
newData();

cd->setRealImage(NULL);
cd->setFFTImage(d1);
cd->setImParm(400.0,4.1,1.0/((r0+n)*ds*2.0),.25);

cd->setLName(s);
cd->setFile(s);

cd->setFlags(CDT_STRUC|CDT_HASFFT);
cd->setCol(11,"sqrt(x10)*amp");
updateDList();

QApplication::restoreOverrideCursor();

}


void Control::Mopenclips(const char *fi) { 
EMData *d1,*d2,*d3;
CtfData *cd;
int n;
char *ss;

QString s;
if (fi) s=fi;
else s=QFileDialog::getOpenFileName(dfdir);

if ( s.isNull() ) return;
QFile file(s);


n=fileCount(s);
if (n==0) return;

QApplication::setOverrideCursor( waitCursor );

QProgressDialog progress( "Reading Clips", "Abort", n, this, 0 ,TRUE );
progress.setProgress( 0 );

d1=new EMData;
d1->readImage(s,0);
ss=(char *)malloc(strlen(d1->Name())+1);
strcpy(ss,d1->Name());
d1->applyMask(-1,6);
d1->normalize();
d3=d1->doFFT();
d1->gimmeFFT();
d3->multConst(d3->ySize());
d3->ri2ap();

for (int i=1; i<n; i++) {
	d1->readImage(s,i);
	d1->applyMask(-1,6);
	d1->normalize();
	d2=d1->doFFT();
	d1->gimmeFFT();
	d2->multConst(d2->ySize());
	d3->addIncoherent(d2);
	delete d2;
	progress.setProgress( i );
	if ( progress.wasCancelled() ) n=i;
}

delete d1;
d3->multConst(1.0/sqrt((float)n));
//d3->normalize();
//d3->ri2ap();
//d3->writeMRC("xxx.img");

cd=new CtfData(d3->ySize()>250?250:d3->ySize());
datalist->insertItem(cd);
cd->copy(current());
datalist->setCurrentItem(cd);
newData();

if (ss[0]=='!'&&ss[1]=='-') cd->setParmStr(ss);
free(ss);

cd->setRealImage(NULL);
cd->setFFTImage(d3);

cd->setLName(s);
cd->setFile(s);
cd->filmno=s.mid(s.findRev('/',-1)+1);
cd->filmno.truncate(cd->filmno.findRev('.',-1));

updateDList();

QApplication::restoreOverrideCursor();

}

void Control::MopenclipsS() { 
EMData *d1,*d2,*d3;
CtfData *cd;
int n,m,j;
char *ss;

QString s( QFileDialog::getOpenFileName(dfdir) );
if ( s.isNull() ) return;
QFile file(s);

m=QMessageBox::information( this, "Ctfit","Select how many particles you'd like\n"
	"In each group?","50", "100", "1/9 total",2,0 );

QApplication::setOverrideCursor( waitCursor );

n=fileCount(s);
if (n==0) return;

if (m==0) m=50;
else if (m==1) m=100;
else m=n/9+1;

QProgressDialog progress( "Reading Clips", "Abort", n, this, 0 ,TRUE );
progress.setProgress( 0 );

for (j=0; j<n/m; j++) {
d1=new EMData;
d1->readImage(s,j*m);
ss=(char *)malloc(strlen(d1->Name())+1);
strcpy(ss,d1->Name());
d1->applyMask(-1,6);
d1->normalize();
d3=d1->doFFT();
d1->gimmeFFT();
d3->multConst(d3->ySize());

for (int i=j*m+1; i<(j+1)*m; i++) {
	d1->readImage(s,i);
	d1->applyMask(-1,6);
	d1->normalize();
	d2=d1->doFFT();
	d1->gimmeFFT();
	d2->multConst(d2->ySize());
	d3->addIncoherent(d2);
	delete d2;
	progress.setProgress( i );
	if ( progress.wasCancelled() ) n=i;
}

delete d1;
d3->multConst(1.0/sqrt((float)m));

cd=new CtfData(d3->ySize()>250?250:d3->ySize());
datalist->insertItem(cd);
cd->copy(current());
datalist->setCurrentItem(cd);
newData();

if (ss[0]=='!'&&ss[1]=='-') cd->setParmStr(ss);
free(ss);

cd->setRealImage(NULL);
cd->setFFTImage(d3);

QString t;
t.sprintf("(%d-%d)",j*m,(j+1)*m-1);
if (s.contains('/')) t.prepend(s+(s.length()-s.findRev('/')));
else t.prepend(s);

cd->setFile(s);
cd->setLName(t);
printf("%s read\n",(const char *)t);
}

updateDList();

QApplication::restoreOverrideCursor();

}

void Control::MopenclipsC() { 
EMData *d1,*d2,*d3;
CtfData *cd;
int n,m,i,j,nc=0;
char *ss,t[80];
float ldf;

QString s( QFileDialog::getOpenFileName(dfdir) );
if ( s.isNull() ) return;
QFile file(s);

QApplication::setOverrideCursor( waitCursor );

n=fileCount(s);
if (n==0) return;

QProgressDialog progress( "Reading Clips", "Abort", n, this, 0 ,TRUE );
progress.setProgress( 0 );

for (j=0; j<n; j++) {
sprintf(t,"ctfsplit.%02d.lst",nc);
d1=new EMData;
d1->readImage(s,j);
if (!d1->hasCTF()) { printf("This data has no CTF parameters set\n"); break; }
d1->writeLST(t,-1);
ldf=*(d1->getCTF());
ss=(char *)malloc(strlen(d1->Name())+1);
strcpy(ss,d1->Name());
d1->applyMask(-1,6);
d1->normalize();
d3=d1->doFFT();
d1->gimmeFFT();
d3->multConst(d3->ySize());

for (i=j+1; i<n; i++) {
	d1->readImage(s,i);
	d1->writeLST(t,-1);
	if (*(d1->getCTF())!=ldf) break;	// new defocus, start new group
	d1->applyMask(-1,6);
	d1->normalize();
	d2=d1->doFFT();
	d1->gimmeFFT();
	d2->multConst(d2->ySize());
	d3->addIncoherent(d2);
	delete d2;
	progress.setProgress( i );
	if ( progress.wasCancelled() ) n=i;
}

delete d1;
d3->multConst(1.0/sqrt((float)i-j));

cd=new CtfData(d3->ySize()>250?250:d3->ySize());
datalist->insertItem(cd);
cd->copy(current());
datalist->setCurrentItem(cd);
newData();

if (ss[0]=='!'&&ss[1]=='-') cd->setParmStr(ss);
free(ss);

cd->setRealImage(NULL);
cd->setFFTImage(d3);

QString t;
t.sprintf("(%d-%d)",j,i);
if (s.contains('/')) t.prepend(s+(s.length()-s.findRev('/')));
else t.prepend(s);

cd->setFile(s);
cd->setLName(t);
printf("%s read\n",(const char *)t);
j=i;
nc++;
}

updateDList();

QApplication::restoreOverrideCursor();

}

// open a clip set with oversampling
void Control::Mopenclipsos() { 
EMData *d1,*d2,*d3,*dc;
CtfData *cd;
int n,os,sz;
char *ss;

QString s( QFileDialog::getOpenFileName(dfdir) );
if ( s.isNull() ) return;
QFile file(s);


n=fileCount(s);
if (n==0) return;

os=QMessageBox::information(this,"Query","How much oversampling would you like ?",
	"2x","3x","4x");
os+=2;

QApplication::setOverrideCursor( waitCursor );

QProgressDialog progress( "Transforming Clips", "Abort", n, this, 0 ,TRUE );
progress.setProgress( 0 );

d1=new EMData;
d1->readImage(s,0);
ss=(char *)malloc(strlen(d1->Name())+1);
strcpy(ss,d1->Name());
d1->applyMask(-1,6);
d1->normalize();
sz=d1->ySize();
dc=d1->clip(-sz*(os-1)/2,-sz*(os-1)/2,sz*os,sz*os);
//dc->normalize();
d3=dc->doFFT();
dc->gimmeFFT();
delete dc;
d3->multConst(d3->ySize());

for (int i=1; i<n; i++) {
	d1->readImage(s,i);
	d1->applyMask(-1,6);
	d1->normalize();
	dc=d1->clip(-sz*(os-1)/2,-sz*(os-1)/2,sz*os,sz*os);
//	dc->writeIMAGIC("tst.img",-1);
//	dc->normalize();
	d2=dc->doFFT();
	dc->gimmeFFT();
	d2->multConst(d2->ySize());
	d3->addIncoherent(d2);
	delete d2;
	delete dc;
	progress.setProgress( i );
	if ( progress.wasCancelled() ) n=i;
}

delete d1;
// This insures that the normalization is always the same
// The 'os' is to compensate for the empty padded area with no energy
// The 1.12... is sqrt(4/PI) to compensate for the circular mask
//d3->multConst(os*1.128379/sqrt((float)n));	
d3->multConst(os/sqrt((float)n));	


//d3->normalize();
//d3->ri2ap();
//d3->writeMRC("xxx.img");

cd=new CtfData(d3->ySize()>250?250:d3->ySize());
datalist->insertItem(cd);
cd->copy(current());
datalist->setCurrentItem(cd);
newData();

if (ss[0]=='!'&&ss[1]=='-') cd->setParmStr(ss);
free(ss);

cd->setRealImage(NULL);
cd->setFFTImage(d3);

cd->setLName(s);
cd->setFile(s);

updateDList();

QApplication::restoreOverrideCursor();

}

void Control::Msavefft() {
if (!current()) return;
if (!current()->getFFTImage()) {
	warn("This entry doesn't have a FFT image!");
	return;
}
QString s( QFileDialog::getSaveFileName() );
if ( s.isNull() ) return;

current()->getFFTImage()->writeMRC(s);

}

void Control::Msaveavg() {
if (!current()) return;

QString s( QFileDialog::getSaveFileName() );
if ( s.isNull() ) return;

CtfData *cur;
DataSet *ds = current()->getData();
Column *col = ds->getColumn(10);

int i,ii,n = ds->rows();
float *d = (float *)calloc(sizeof(float),n);
float *d2;

for (ii=0; cur=(CtfData *)datalist->item(ii); ii++) {
	if (!(cur->flags()&CDT_PLOT)) continue;
	ds=cur->getData();
	if (ds->rows()!=n) continue;
	col=ds->getColumn(14);
	d2=col->getData();
	for (i=0; i<n; i++) d[i]+=d2[i];
}

col=ds->getColumn(0);
save_data(ds->dataAt(0,0),ds->dataAt(0,1)-ds->dataAt(0,0),d,n,(const char *)s);
free(d);
}

void Control::Msavedata() {
if (!current()) return;
QString s( QFileDialog::getSaveFileName() );
if ( s.isNull() ) return;

current()->getData()->writeFile((const char *)s);

}

// This saves 1 column from multiple data sets in a single spreadsheet
// Only works properly if all data sets have the same range of s
void Control::Msavedata1c() {
if (!current()) return;

QString s( QFileDialog::getSaveFileName() );
if ( s.isNull() ) return;

QString cols = OneLiner::getOneLine("Enter column number\n3 - CTF & Envel (amp)\n5 - Noise (Inten)\n6 - Complete (Inten)\n\
8 - Contrast (SNR)\n10 - Image Pow Spec", "6",this);
int coln = cols.toInt();
if (coln<1||coln>14) return;

CtfData *cur;
DataSet *ds = current()->getData();
Column *col;

int i,ii,n = ds->rows();
float *d2;

FILE *out=fopen((const char *)s,"w");
if (!out) return;

for (i=0; i<n; i++) {
	fprintf(out,"%1.5f",ds->dataAt(0,i));
	for (ii=0; cur=(CtfData *)datalist->item(ii); ii++) {
		if (!(cur->flags()&CDT_PLOT)) continue;
		ds=cur->getData();
		if (ds->rows()!=n) continue;
		fprintf(out,"\t%1.5g",ds->dataAt(coln,i));
	}
	fprintf(out,"\n");
}

fclose(out);
}

void Control::Mprint() { plotwin->print(); }

void Control::Mquit() { 
// This restores the window geometries
FILE *out = DFopenw("ctfitwin");
if (out) {
	fprintf(out,"%d %d %d %d\n",x(),y(),width(),height());
	fprintf(out,"%d %d %d %d\n",plotwin->x(),plotwin->y(),plotwin->width(),plotwin->height());
	fprintf(out,"%d %d %d %d\n",imgwin->x(),imgwin->y(),imgwin->width(),imgwin->height());
	fclose(out);	
}

saveMem(); 
exit(0); 
}

void Control::Mopenclipscf() { warn("Open carbon film clips not supported yet!"); }

void Control::MWFilter() {

if (current()) current()->WFilter();

}
void Control::Mfixset() { 

if (current()) current()->fixPhase();

}

void Control::Msetparm() { 

if (current()) current()->setParm();

}

void Control::Mpair() { warn("Focal pair merging not implimented in CTFIT yet, see the 'focalpair' program."); }

void Control::MrtCTF() {
if (!rtwin) {
	rtwin=new Image;
	rtwin->resize(100,100);
}
else {
	delete rtwin->getData()->Parent();
	delete rtwin->getData();
	rtwin->setData(NULL);
}
rtwin->show();

QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

EMData *rtimg = new EMData();
rtimg->readImage(s,0);
rtwin->setData(rtimg->copy());

}

void Control::Mapply() { 
if (current()) current()->applyCtf();
}

void Control::MWFilterMRC() { 
if  (!current()) return;

QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

if (current()) current()->WFilterMRC(s);
}

void Control::Mapply3d() { 
if  (!current()) return;

QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

current()->applyCtfTo(s,1);
}

void Control::Mapply3dn() { 
if  (!current()) return;

QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

current()->applyCtfTo(s,2);
}

void Control::MsetSF() { 
if  (!current()) return;

QString s( QFileDialog::getSaveFileName() );
if ( s.isNull() ) return;

current()->setSF(s);
}

void Control::Munapply() { 
if  (!current()) return;

QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

current()->applyCtfTo(s,-1);
}

void Control::Mnoise() { 
if (current()) current()->applyCtf(1);
}

void Control::MshowEq() {
if (eqed) { eqed->show(); return; }

eqed=new EqEd();
connect(eqed,SIGNAL(selCol(int)),this,SLOT(needColEq(int)));
connect(eqed,SIGNAL(setEq(int,QString)),this,SLOT(setColEq(int,QString)));
connect(eqed,SIGNAL(newVal(float,float,float,float,float,float)),this,SLOT(setQVals(float,float,float,float,float,float)));
eqed->show();
needColEq(1);
}

void Control::needColEq(int c) {
if (!current()) { eqed-> setText(""); return; }
if (c>=0) {
	DataSet *ds = current()->getData();
	Column *col = ds->getColumn(c);
	eqed->setText(col->getEqn());
}
else {
	eqed->setText(current()->getAEq(c));
}
//printf("restore %d -> %s\n",c,(const char *)col->getEqn());
}

void Control::setColEq(int c ,QString eq) {
if (!current()) return;

DataSet *ds;
Column *col;
CtfData *cur;
int ii;
for (ii=0; cur=(CtfData *)datalist->item(ii); ii++) {
	if (!(cur->flags()&CDT_PLOT)) continue;
	if (c>=0) {
		ds=cur->getData();
		col=ds->getColumn(c);
		col->setEqn((const char *)eq,current()->steps(),ds); 
	}
	else {
		cur->setAEq(c,eq);
	}
//printf("set %d -> %s\n",c,(const char *)eq);
}

}

void Control::setQVals(float q1,float q2,float q3,float q4,float q5,float q6) {
if (!current()) return;
//printf("Q %f %f %f %f %f %f\n",q1,q2,q3,q4,q5,q6);
current()->setQ(q1,q2,q3,q4,q5,q6);
current()->updateEqn();
}

void Control::Mseteqn() {
if (!current()) return;
DataSet *ds = current()->getData();
Column *col = ds->getColumn(14);

QString neq = OneLiner::getOneLine("Enter a new equation using:\n\
x0=s, x1=gamma, x3=ctf amp, x5=noise inten,\n\
x10=img pow spec, x11=calc SF, x13=SF from file\n\
also Cs,lambda,defocus,B,amp,ampcont,exps4,noise(1,3,2,4)\n",
	 col->getEqn());

if (!neq.isEmpty()) {
	CtfData *cur;
	for (int ii=0; cur=(CtfData *)datalist->item(ii); ii++) {
		if (!(cur->flags()&CDT_PLOT)) continue;
		ds=cur->getData();
		col=ds->getColumn(14);
		col->setEqn((const char *)neq,current()->steps(),ds); 
	}
}

}

void Control::Mhelp() {
	char s[512];
	sprintf(s,"netscape -remote 'openFile(%s/doc/progs/ctfit.html,new-window)'",
		EMDIR());
	run(s);
}

void Control::Mabout() { 
	QMessageBox::about(this,"About CTFIT",
 "CTFIT\nCopyright 1998 Steve Ludtke\n\n\
 Ctfit is designed to simulate transmission\n\
 electron microscope CTFs as well as determine\n\
 CTF parameters from micrographs. It is then\n\
 capable of doing phase correction of images\n\
 and storing parameters for external CTF\n\
 correction software.");
}

void Control::warn(char *msg) {
	QMessageBox::warning(this,"CTFIT Warning",msg,"Ok");
	return;
}

void Control::closeEvent(QCloseEvent *e) {
	e->accept();
	Mquit();
} 

// when recalculation is required
void Control::parmchange(float value) {
if (!current()) return;
float B;

B=envel->Value();
if (!envbflag) B=pow(B*PI/2.0,2.0);

current()->setImParm(atof(voltage->text()),atof(Cs->text()),atof(apix->text()),.5/atof(apix->text()));	
current()->setSigParm(-df->Value(),B, amp->Value(),ampc->Value()/100.0,exps4->Value());
current()->setNoiseParm(expa->Value(),expw->Value(),gaussa->Value(),gaussw->Value());
current()->setStigParm(stigang->Value(),stigdf->Value());
current()->setStrucParm(cfilmsf->Value());
current()->updateEqn();

//imgwin->setRad(current()->getZeroR(),current()->getZeroStigR(),current()->stigang);
ImShape *s=imgwin->shapeAt(0);
if (s) {
	s->userf=current()->stigang;
	s->x0=current()->getZeroR()*imgwin->lscale;
	s->y0=current()->getZeroStigR()*imgwin->lscale;
//printf("x %d %d %1.2f\n",s->x0,s->y0,s->userf);
	imgwin->update();
}
else printf("Error, no shape\n");

if (rtwin) {
	current()->applyCtf(rtwin->getData());
	rtwin->imageChanged();
}

if (parmDisp) {
	parmDisp->setText(current()->getParm());
}
//plotwin->newplots();
//plotwin->replot();
}

// when V or Cs change
void Control::vchange() {
parmchange(0);
}

// when A/pix changes
void Control::apixchange() {
parmchange(0);
}

// when mag or scan change
void Control::magchange() {
if (!current()) return;
float Mag,Scan;
char Apix[40];
QString str;

str=OneLiner::getOneLine("Enter Mag (1000x),scan step (um)",",60,14",NULL);

if (sscanf((const char *)str," %f , %f",&Mag,&Scan)!=1) return;

if (current()->mag<1.0) current()->mag=1.0;
//if (current()->mag>500.0) current()->mag=500.0;
sprintf(Apix,"%1.3f",Scan*10.0/Mag);
apix->setText(Apix);

parmchange(0);
}

void Control::dataNewsim() {
CtfData *d;

d=new CtfData();
d->copy(current());
datalist->insertItem(d);
datalist->setCurrentItem(d);

updateDList();
}

void Control::dataDel() {
if (!current()) return;
delete current();
updateDList();
}

void Control::newDataName() {

if (!current()) return;
current()->setLName(dataname->text());
updateDList();
}

// When a new data set is created this makes sure the interface is consistent
// the new data set should be set to 'current'
void Control::newData() {
char s[20];

if (current()==NULL) return;

df->setValueQ(-current()->defocus);
if (envbflag) envel->setValueQ(current()->B);
else envel->setValueQ(sqrt(current()->B)*2.0/PI);
amp->setValueQ(current()->amp);
ampc->setValueQ(current()->ampcont*100.0);
exps4->setValueQ(current()->exps4);
expa->setValueQ(current()->noise1);
expw->setValueQ(current()->noise2);
gaussa->setValueQ(current()->noise3);
gaussw->setValueQ(current()->noise4);
stigang->setValueQ(current()->stigang);
stigdf->setValueQ(current()->stigdf);

cfilmsf->setValueQ(current()->cfilmsf);

sprintf(s,"%d",(int)floor(current()->V+.5));
voltage->setText(s);

sprintf(s,"%3.1f",current()->Cs);
Cs->setText(s);

sprintf(s,"%1.3f",current()->sstep);
apix->setText(s);

sprintf(s,"%1.1f",1.0/current()->smax);
// maxres->setText(s);

newMicroscope(current()->microscope);

if (current()->structype==-1) sffile->setChecked(1);
else if (current()->structype==1) sfsph->setChecked(1);
else sfnone->setChecked(1);

//plotwin->setCurrent(current);
//plotwin->newplots();

//imgwin->setData(current()->getFFTImage());

//dataname->setText(current()->getText());
}

void Control::dataHi(int index) {
char s[20];

if (current()==NULL) return;

df->setValueQ(-current()->defocus);
if (envbflag) envel->setValueQ(current()->B);
else envel->setValueQ(sqrt(current()->B)*2.0/PI);
amp->setValueQ(current()->amp);
ampc->setValueQ(current()->ampcont*100.0);
exps4->setValueQ(current()->exps4);
expa->setValueQ(current()->noise1);
expw->setValueQ(current()->noise2);
gaussa->setValueQ(current()->noise3);
gaussw->setValueQ(current()->noise4);
stigang->setValueQ(current()->stigang);
stigdf->setValueQ(current()->stigdf);
cfilmsf->setValueQ(current()->cfilmsf);

sprintf(s,"%d",(int)floor(current()->V+.5));
voltage->setText(s);

sprintf(s,"%3.1f",current()->Cs);
Cs->setText(s);

sprintf(s,"%1.3f",current()->sstep);
apix->setText(s);

newMicroscope(current()->microscope);

if (current()->structype==-1) sffile->setChecked(1);
else if (current()->structype==1) sfsph->setChecked(1);
else sfnone->setChecked(1);

plotwin->setCurrent(current());
plotwin->newplots();

imgwin->setData(current()->getFFTImage());

dataname->setText(current()->lName());


}

void Control::doFit() {
current()->doFit();
dataHi(-1);
}

void fitQ(QList<CtfData> *);

void Control::doFitX() {

}

void Control::doFitQ() {
QList<CtfData> dsel;
CtfData *cur;
int ii;

for (ii=0; cur=(CtfData *)datalist->item(ii); ii++) {
	if (cur->flags()&CDT_PLOT && cur->flags()&CDT_HASFFT) dsel.append(cur);
}

fitQ(&dsel);

dataHi(-1);
}

void Control::newS(QMouseEvent *e) {
float s,ang;
float x,y;

if (!current()) return;

if (e->state() & LeftButton ) { 
	x=imgwin->pxtodx(e->x())-current()->getFFTImage()->xSize()/2.0;
	y=imgwin->pytody(e->y())-current()->getFFTImage()->ySize()/2.0;
	s=hypot(x,y);
	ang=atan2(y,x)*180.0/PI;

	current()->setSZero(s,-1.0,ang);
	stigdf->setValueQ(current()->stigdf);
	stigang->setValueQ(ang);
	df->setValue(-current()->defocus);
//	current()->updateEqn();
}
}

void Control::dataSel(int index) {
int i;

//printf("datasel %d %d\n",index,lastdata);

i=current()->flags();
i^=CDT_PLOT;
current()->setFlags(i);
//if (!(i&CDT_PLOT)) {  current=data->at(lastdata); }

updateDList();
}

void Control::setmemFile() {
QString s( QFileDialog::getSaveFileName() );
if ( s.isNull() ) return;

saveMem();
curmemfile=s;
initMem();
}

void Control::initMem() {
FILE *in=fopen((const char *)curmemfile,"r");
mem.clear();
if (!in) {
	error(ERR_WARN,"Cannot open CTF parameter file");
	CtfParm *p = new CtfParm;
	mem.append(p);
	return;
}

CtfParm  tst;
if (tst.read(in)) {
	rewind(in);
	CtfParm *p;
	while (p=new CtfParm,p->read(in)) mem.append(p);
	delete p;
	curmemfile+=".new";
}
else if (tst.newread(in)) {
	rewind(in);
	CtfParm *p;
	while (p=new CtfParm,p->newread(in)) mem.append(p);
	delete p;
}
else {
	error(ERR_WARN,"Don't understand the data in CTF parameter file");
	mem.clear();
	CtfParm *p = new CtfParm;
	mem.append(p);
	return;
}

memRcl(0);
updateMList();
return;
}

void Control::saveMem() {
if (mem.count()==0) return;

FILE *out = fopen(curmemfile,"w");

if (!out) {
	error(ERR_WARN,"Cannot open output CTF file");
	return;
}

for (int i=0; i<mem.count(); i++) mem.at(i)->newwrite(out);

fclose(out);
}

void Control::setsfnone() {

current()->setStrucMode(0,QString());
parmchange(0);
}

void Control::setsfsph() {
current()->setStrucMode(1,QString());
parmchange(0);
}

void Control::setsffile() {
QString fsp(QFileDialog::getOpenFileName (current()->strucfile));
if (fsp.isNull()) return;

current()->setStrucMode(-1,fsp);
parmchange(0);
}

void Control::updateMList() {
int i;
char s[80];

memlist->setAutoUpdate(FALSE);
memlist->clear();
for (i=0; i<mem.count(); i++) {
	memlist->insertItem(mem.at(i)->name);
}
if (curmem>=0) memlist->setCurrentItem(curmem);
memlist->setAutoUpdate(TRUE);
memlist->update();

}


void Control::memSet() {
if (!current()) return;
CtfParm *p = mem.at(curmem>=0?curmem:0);

p->defocus=current()->defocus;
p->B=current()->B;
p->amp=current()->amp;
p->ampcont=current()->ampcont;
p->exps4=current()->exps4;
p->noise1=current()->noise1;
p->noise2=current()->noise2;
p->noise3=current()->noise3;
p->noise4=current()->noise4;
p->stigang=current()->stigang;
p->stigdf=current()->stigdf;
p->cfilmsf=current()->cfilmsf;
p->structype=current()->structype;
p->strucfile=current()->strucfile;
p->V=current()->V;
p->Cs=current()->Cs;
p->sstep=current()->sstep;

p->microscope=microscope->currentText();

saveMem();
}

void Control::memDel() {
if (curmem>=0) mem.remove(curmem);
updateMList();
}

void Control::memRcl() { memRcl(0); }

void Control::memRcl(int n) {
CtfParm *parm;
if (!current()) return;

//printf("memrcl %d\n",n);
parm=mem.at(curmem);
if (!parm) return;
current()->microscope=parm->microscope;
current()->setImParm(parm->V,parm->Cs,parm->sstep,1.0/(parm->sstep*2));	
current()->setSigParm(parm->defocus,parm->B,parm->amp,parm->ampcont,parm->exps4);
current()->setNoiseParm(parm->noise1,parm->noise2,parm->noise3,parm->noise4);
current()->setStrucParm(parm->cfilmsf);
current()->setStrucMode(parm->structype,parm->strucfile);
current()->setStigParm(parm->stigang,parm->stigdf);

current()->updateEqn();
dataHi(-1);
}

void Control::memNew() {
curmem=mem.count();
mem.append(new CtfParm);
memSet();
updateMList();
}

void Control::memHi(int n) {
if (n<0) n=0;
//printf("Memhi %d\n",n);
curmem=n;
memname->setText(mem.at(n)->name);
return;
}

void Control::newMemName() {
if (curmem<0) return;
mem.at(curmem)->name=memname->text();
updateMList();
}

CtfParm::CtfParm() {
name="Default";
comment="";
microscope="";
scanner="";
specimen="";

defocus=-1.5;
B=400.0;
amp=1.0;
ampcont=.1;
exps4=0;
noise1=0;
noise2=0;
noise3=0;
noise4=0;
stigang=0;
stigdf=0;
cfilmsf=300.0;
structype=0;
V=100.0;
Cs=5.6;
sstep=4.66667;
}

int CtfParm::oldread(FILE *in) {
char c[40];
float smax;

fgets(c,39,in);
c[39]=0;
if (c[0]!=0) c[strlen(c)-1]=0;
name=c;
if (fscanf(in," %f %f %f %f %f %f %f %f %f %f %f %f %d %f %f %f %f",&defocus,&B,
        &amp,&ampcont,&exps4,&noise1,&noise2,&noise3,&noise4,
        &stigang,&stigdf,&cfilmsf,&structype,&V,&Cs,&sstep,&smax)!=17) return 0;
fgets(c,4,in);

mag=140.0/sstep;	//assume default of 14um pixel
comment="";
microscope="";
scanner="";
specimen="";
filmno=name;
dose=0;
date=QDate::currentDate();

return 1;
}

int CtfParm::newread(FILE *in) {
char s[512],t[128],u[128];
int i;


u[0]=0;
s[0]=0;
fgets(s,511,in);
while (s[0]=='#') fgets(s,511,in);

if (sscanf(s,"%s %f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%s",t,&defocus,&B,&amp,&ampcont,&noise1,&noise2,&noise3,&noise4,
	&V,&Cs,&sstep,u)<12) return 0;
name=t;
strucfile=u;
if (strlen(t)>1) structype-=1; else structype=0;
return 1;
}

int CtfParm::newwrite(FILE *out) {
fprintf(out,"%s\t%1.6g,%1.6g,%1.6g,%1.6g,%1.6g,%1.6g,%1.6g,%1.6g,%1.6g,%1.6g,%1.6g,%s\n",(const char *)name,defocus,B,amp,ampcont,noise1,
	noise2,noise3,noise4,V,Cs,sstep,(const char *)strucfile);
	
return 1;
}

int CtfParm::read(FILE *in) {
char c[512];
int m,d,y,i;

fgets(c,39,in);
c[39]=0;
c[strlen(c)-1]=0;
name=c;
fgets(c,511,in);
if ((i=sscanf(c," %f %f %f %f %f %f %f %f %f %f %f %f %d %f %f %f %f %d %f %d %d %d",&defocus,&B,
	&amp,&ampcont,&exps4,&noise1,&noise2,&noise3,&noise4,
	&stigang,&stigdf,&cfilmsf,&structype,&V,&Cs,&sstep,&mag,&nptcl,&dose,&y,&m,&d))<18) return 0;
//printf("%d %d %d %d\n",i,y,m,d);
if (ampcont>1.0&&ampcont<=100.0) ampcont/=100.0;
if (i>=22) date.setYMD(y,m,d); else date=QDate::currentDate();
if (i<19) dose=0;
fgets(c,128,in);
comment=c;
comment=comment.stripWhiteSpace();
fgets(c,128,in);
microscope=c;
microscope=microscope.stripWhiteSpace();
fgets(c,128,in);
scanner=c;
scanner=scanner.stripWhiteSpace();
fgets(c,128,in);
specimen=c;
specimen=specimen.stripWhiteSpace();
fgets(c,128,in);
filmno=c;
if ((i=filmno.find('\t'))>=0) {
	strucfile=filmno.mid(i+1);
	strucfile=strucfile.stripWhiteSpace();
	filmno=filmno.left(i);
}
filmno=filmno.stripWhiteSpace();
//if (structype==-1) structype=0;		// we don't store the name of a structure factor file

return 1;
}

void CtfParm::write(FILE *out) {
int i;

fputs((const char *)name,out);
fprintf(out,"\n%f %f %f %f %f %f %f %f %f %f %f %f %d %f %f %f %f %d %1.2f %d %d %d\n",defocus,B,
	amp,ampcont,exps4,noise1,noise2,noise3,noise4,
	stigang,stigdf,cfilmsf,structype,V,Cs,sstep,mag,nptcl,dose,date.year(),date.month(),date.day());

if ((i=comment.find('\n')) || (i=comment.find('\r'))) comment.truncate(i);
if ((i=microscope.find('\n')) || (i=microscope.find('\r'))) microscope.truncate(i);
if ((i=scanner.find('\n')) || (i=scanner.find('\r'))) scanner.truncate(i);
if ((i=specimen.find('\n')) || (i=specimen.find('\r'))) specimen.truncate(i);
if ((i=filmno.find('\n')) || (i=filmno.find('\r'))) filmno.truncate(i);
comment.truncate(127);
microscope.truncate(127);
scanner.truncate(127);
specimen.truncate(127);
filmno.truncate(10);

fprintf(out,"%s\n%s\n%s\n%s\n%s",(const char *)comment,(const char *)microscope,
	(const char *)scanner,(const char *)specimen,(const char *)filmno);
if (structype==-1) fprintf(out,"\t%s",(const char *)strucfile);
fprintf(out,"\n");
}

