#include "InspFile.h"
#include "ImageMx.h"
#include "qmessagebox.h"
#include "qprogressdialog.h"

#define Inherited InspFileData

InspFile::InspFile(QWidget* parent,const char* name):Inherited( parent, name ) {
cdat=NULL;
cnum=cdnum=tnum=0;
detached=0;

//connect(brite,SIGNAL(valueChanged(int)),this,SLOT(newBCZ(int)));
//connect(cont,SIGNAL(valueChanged(int)),this,SLOT(newBCZ(int)));
//connect(imview,SIGNAL(mousePress(QMouseEvent *)),this,SLOT(imClick(QMouseEvent *)));
connect(&tmr,SIGNAL(timeout()),this,SLOT(timed()));
connect(big,SIGNAL(clicked()),this,SLOT(bigpress()));
connect(detach,SIGNAL(clicked()),this,SLOT(detachpress()));
//connect(more,SIGNAL(clicked()),this,SLOT(morepress()));
connect(imgn,SIGNAL(valueChanged(int)),this,SLOT(setN(int)));
tmr.start(190);
//setBackgroundMode(NoBackground);
}

InspFile::~InspFile() {
if (cdat) delete cdat;
}

void InspFile::setFile(const char *fi) {
char s[100];
int r;

file=fi;

imview->setNullString("No Image");
//imview->setData(NULL);
if (cdat) delete cdat;

cdat=new EMData;
tnum=fileCount(file);
r=cdat->readImage(file,0,-1);
if (tnum==cdat->xSize()) r=cdat->readImage(file,-1,-1);

if (r) {
	type->setText("INVALID IMAGE FILE");
	title->setText("");
	count->setText("");
	euler->setText("");
	imview->setData(NULL);
	return;
}

// image type
if (cdat->getMRCHed()) type->setText("MRC File");
else if (cdat->getImagicHed()) {
	if (cdat->zSize()>1) type->setText("Imagic3d File");
	else type->setText("Imagic File");
}
else if (cdat->getSpiHed()) {
	type->setText("Spider File");
}

// number of images
if (tnum<=1) count->setText("Single Image");
else {
	sprintf(s,"%d Images",tnum);
	count->setText(s);
}
imgn->setRange(0,tnum-1);

// image size
if (cdat->zSize()==1) sprintf(s,"%dx%d pixels",cdat->xSize(),cdat->ySize());
else sprintf(s,"%dx%dx%d voxels",cdat->xSize(),cdat->ySize(),cdat->zSize());
size->setText(s);

// title
title->setText(cdat->Name());

// Euler,etc...
sprintf(s,"Euler: %1.2f,%1.2f,%1.2f",cdat->alt()*180.0/PI,cdat->az()*180.0/PI,cdat->phi()*180.0/PI);
euler->setText(s);

if (cdat->NImg()>1) sprintf(s,"From %d raw image(s)",cdat->NImg());
else strcpy(s,"");
from->setText(s);

density->setText("");

//display image
if (detached || (cdat->xSize()<320 && cdat->ySize()<320)) {
	delete cdat;
	cdat=new EMData;
	cnum=-1;
	cdnum=imgn->value();
	if (!detached) imview->setNullString("Loading");
//	dispImg();
//	cdat->readImage(file,0);
//	imview->setData(cdat);
}
else {
	imview->setData(NULL);
	imview->setNullString("'Big' View Required");
}

}

void InspFile::imClick(QMouseEvent *) {
	if (cdat->getFlags()&EMDATA_NODATA) return;
	
	cdnum++;
	cdnum=cdnum%tnum;
}

void InspFile::setN(int n) {
if (cdat->getFlags()&EMDATA_NODATA) return;
cdnum=n;

}

void InspFile::dispImg() {
	if (!cdat) imview->setData(NULL);
	if (!cdat || cnum==cdnum) return;
	cnum=cdnum;

	if (cdat->readImage(file,cnum)==0) {
		char s[80];
		// Euler,etc...
		sprintf(s,"Euler: %1.2f,%1.2f,%1.2f",cdat->alt()*180.0/PI,cdat->az()*180.0/PI,cdat->phi()*180.0/PI);
		euler->setText(s);

		if (tnum>1) {
			sprintf(s,"%d of %d Images",cnum+1,tnum);
			count->setText(s);
		}

		if (cdat->NImg()>1 && cdat->NImg()<100000) sprintf(s,"From %d raw image(s)",cdat->NImg());
		else strcpy(s,"");
		from->setText(s);

		sprintf(s,"Den: %1.3g,%1.3g  Avg=%1.3g Sig=%1.3g",cdat->Min(),cdat->Max(),cdat->Mean(),cdat->Sigma());
		density->setText(s);

		title->setText(cdat->Name());

//		cdat->normalize();
		imview->setData(cdat);
	}
	else imview->setData(NULL);
}

void InspFile::timed() {

if (cnum!=cdnum && cdat) dispImg();

}

void InspFile::detachpress() {
delete imview;
detached^=1;
if (detached) {
	imview = new Image(NULL, "User_5");
	imview->setGeometry(600, 200, 250, 250);
	imview->setScale(1.0);
}
else {
	imview = new Image(this, "User_5");
	imview->setGeometry(425, 5, 140,140);
}
imview->setMinimumSize(50, 50);
imview->setMaximumSize(2048, 2048);
imview->setFocusPolicy(QWidget::NoFocus);
imview->setBackgroundMode(QWidget::NoBackground);
imview->setBackgroundOrigin( QWidget::WidgetOrigin );
imview->show();
setFile(file);
}

void InspFile::bigpress() {
int n=fileCount(file);
int td=0,k=1,i;

if (n==0) return;

ImageMx *imx;

imx=new ImageMx;
imx->setAutoDelete(TRUE);

EMData *d;
QProgressDialog progress( "Reading images", "Abort", n, NULL, 0 ,TRUE );
progress.setProgress( 0 );

for (i=0; i<n; i++) {
	if (td>20000000 && k) {
		k=QMessageBox::information( this, "eman","Over 20 megabytes read so far\n\
Continue reading images ?","&Yes", "&No");
		if (k==1) break;
	}

	progress.setProgress( i );
	if ( progress.wasCancelled() ) break;

	d=new EMData;
	d->readImage(file,i);
//	d->normalize();
	imx->addData(d);

	td+=d->xSize()*d->ySize()*d->zSize()*4;
}

imx->show();

}

void InspFile::morepress() {
if (file.contains("cls") || file.contains("classes")) emit(showcls(file));
else emit(showtri(file));
}
