// align2d.C 8/99  Steve Ludtke
// Copyright 1999 Baylor College of Medicine

//N align2d
//F This program will align a set of images to a reference
//T LA
//1
//P <fixed input>	Reference image
//P <input>	Images to be aligned
//P <output>	Output filename
//P [noflip]	Rotational/translational alignment without flips
//P [pgm]	Output in pgm format, note there should only be one input in this case
//P [log]	Logs rotations/translations to align.log
//P [edgenorm]	Perform an edgenormalization of both images before alignment
//U align2d proj.hed cls000.hed tst.hed
//D Pretty self explanatory. If the input is a stack, the first image is used.


#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
EMData r,d,*da;
int i,n,pgm=0,noflip=0,edgenorm=0,log=0,fsc=0,noali=0;

appinit(argc,argv);

if (argc<4) appusage(argv[0]);

for (i=4; i<argc; i++) {	
	if (strncmp(argv[i],"noflip",6)==0) noflip=1;
	else if (strncmp(argv[i],"pgm",3)==0) pgm=1;
	else if (strncmp(argv[i],"log",3)==0) log=1;
	else if (strncmp(argv[i],"fsc",3)==0) fsc=1;
	else if (strncmp(argv[i],"noali",3)==0) noali=1;
	else if (strncmp(argv[i],"edgenorm",8)==0) edgenorm=1;
	else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}

n=fileCount(argv[2]);
r.readImage(argv[1],0);
if (edgenorm) r.edgeNormalize();

for (i=0; i<n; i++) {
	d.readImage(argv[2],i);
	if (edgenorm) d.edgeNormalize();
	printf("%d. orig %5.2f, %5.2f, %5.2f\t",i,d.alt(),d.az(),d.phi());
	if (noali) da=d.copy(0,0);
	else if (noflip) da=d.RTAlign(&r);
	else {
		da=d.RTFAlign(&r);
		da->refineAlign(&r);
	}
	printf("dx,dy,da = %1.0f, %1.0f\t%5.2f\n",da->Dx(),da->Dy(),da->alt());
	if (log) {
		FILE *out=fopen("align.log","a");
		if (out) {
			fprintf(out,"%s\t%d\t%d\t%1.2f\t%1.2f\n",argv[2],(int)round(da->Dx()),(int)round(da->Dy()),da->alt()*180/PI,da->ufloat4);
			fclose(out);
		}
	}

	if (fsc) {
		char s[80];
		sprintf(s,"fsc2d.%d.txt",i);
		float *fsc = da->fsc(&r);
		save_data(0,1,fsc,da->ySize()/2,s);
		free(fsc);
	}
	if (pgm) da->writePGM(argv[3],da->Min(),da->Max());
	else da->writeIMAGIC(argv[3],-1);
	delete da;
} 

}
