// msa2.c 1998 Phil Baldwin
/*              msa.C 
    is a variant of pca.c. The acknowledgements for this latter program are
	 appended at the end of this one.
	 
	 msa2.C takes 
	 			a footprint file
		and outputs another
		         (shorter) footprint output file.
				It also takes a different footprint file
				    and reduces it using the same eigenvectors. 
       ***********S****************
         footprint files
			 have the following format
			         120 byte string  --- copied directly to output
					 2 integers    (a)  n - number of images
					               (b)  n2 - number of elements; 32
					 an array of n2 rows of n numbers
					 (optionally) euler angles 

          ****************************
		  the user picks n2'

*/
//N msa2
//F This performs multivariate statistical analysis on 2 sets of invariants
//T LT
//1
//P <input fp 1>	A footprint file to be analyzed
//P <input fp 2>	A footprint file to be analyzed
//P <out foot file 1>	The reduced dimensionality output file
//P <out foot file 2>	The reduced dimensionality output file
//P <final n2 value>	Dimensionality of the output
//U msa file.fp file.msa.fp 8
//D This program will perform multivariate statistical analysis on
//E input 1 (produced by one of the <a href=footprint.html>
//E footprint</a> programs. It then uses the new basis set to
//E reduce the dimensionality of input 2. The output is two footprint files
//E with reduced dimensionality.

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "List.h"
#include "util.h"
#define SIGN(a, b) ( (b) < 0 ? -fabs(a) : fabs(a) )
// #define TESTFILE 0

void corcol(float **data,int n,int m,float **symmat);
void covcol(float **data,int n,int m,float **symmat);
void scpcol(float **data,int n,int m,float **symmat);
void erhand(char err_msg[]);
float *vector(int n);
float **matrix(int n,int m);
void free_vector(float *v,int n);
void free_matrix(float **mat,int n,int m);
void tred2(float **a,int n,float *d,float *e);
void tqli(float d[],float e[],int n,float **z);


int CONVERGE=30;
main(int argc,char *argv[])
{
FILE *footinfile, *footinfile2, *footoutfile, *footoutfile2;
FILE *testfile;
int  n2, n,  n22,i, j, k, k2, m, npart, npart2;
int index, jmax; //PRB
int nmin, nmax;
int CUTOFF, TESTFILE;
float **data, **data2, **symmat, **symmat2, *evals, *interm;
float in_value;
float *column, evalmax; // PRB
float q, stddev;
char option, *strncpy(),s[120], s2[120];

appinit(argc,argv);

//printf ("%d \n",argc);
if (argc !=  6 && argc != 7)
	{
	printf("msa2\n Usage: ");
	printf("msa2 <in foot file> <in fprint file2> <out foot file> <out foot file2> <final n2 value>[CONVERGE=30]\n\n");
	// <n':number of eigenvals> 
	exit(0);
	}

if ((footinfile = fopen(argv[1],"r")) == NULL || (footinfile2 = fopen(argv[2],"r")) == NULL )
	{
	fprintf(stderr, "Program %s : cannot open file %s or file %s\n",argv[0], argv[1], argv[2]);
	fprintf(stderr, "Exiting to system."); exit(0);
	}

if (argc ==7) CONVERGE= atoi(argv[6]);
int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);
LOG(Ref,argv[2],LOG_INFILE,NULL);
LOG(Ref,argv[3],LOG_OUTFILE,NULL);
LOG(Ref,argv[4],LOG_OUTFILE,NULL);

testfile = fopen("test","r"); // PRB
fread(s,1,120,footinfile);
fread(s2,1,120,footinfile2);
fread(&npart,4,1,footinfile);// should read npart
fread(&npart2,4,1,footinfile2);// should read npart
fread(&n2,4,1,footinfile); // should read n2
fread(&n22,4,1,footinfile2); // should read n2
if (n2 != n22) {
	fprintf(stderr, "The length of the footprint for both infiles, %d %d, must be the same \n",n2,n22); 
	exit(0);	}

CUTOFF = atoi(argv[5]);
TESTFILE = 0; //atoi(argv[4]);

printf("No. of rows, equivalently number of features, n2: %d,     final n2: %d.\n",n2,   CUTOFF);
printf("No. of columns, equivalently number of particles: %d.\n",npart);
printf("Input file: %s.\n",argv[1]);
printf(" %c \n",s);
 
nmin = n2; if (npart < nmin) nmin = npart;
nmax = npart + n2-nmin;

printf("nmin: %d, nmax: %d \n",nmin,nmax);

n = npart;  // n will be large; number of particles
m =n2;  //  m will be typically 32, which is n2; 

if (TESTFILE) { n= 4; m =3;}
        
column = (float *) malloc((m+1)*sizeof(float));
 
footoutfile = fopen(argv[3],"w");
fwrite(s,120,1,footoutfile);
fwrite(&npart,4,1,footoutfile);
footoutfile2 = fopen(argv[4],"w");
fwrite(s2,120,1,footoutfile2);
fwrite(&npart2,4,1,footoutfile2);

fwrite(&CUTOFF,4,1,footoutfile);
fwrite(&CUTOFF,4,1,footoutfile2);

// open new file for writing
// read 120 byte string from footinfile
// and write it to footoutfile
//  read in n and n2


// I set the n from pca.c equal to number of rows which should be n2
// I set the m from pca.c equal to n


// CUTOFF = 5;


printf("m: %d, n: %d, CUTOFF: %d.\n",m,n,CUTOFF);
 
    /* Now read in data. */

    data = matrix(n, m);  /* Storage allocation for input data , npart by m  */
    data2 = matrix(npart2, n22);  /* Storage allocation for input data , npart by m  */

       
	printf(" footprint of particles 1, 2,3 4 is \n");
	for (j = 1; j <= m; j++)  {// j labels the footprint
		for (i = 1; i <= n ; i++)  // i labels the particle
// 		The above two lines are different for pca.C  
//      But j and m must correspond to the footprint
//             So as i changes, must be reading in data
            {
		if (TESTFILE) { fscanf(testfile, "%f", &in_value);
			} else {
 		fread(&in_value,sizeof(float),1,footinfile);}
            data[i][j] = in_value;                 // notice data[i][j] is transpose of how 
			if (i<=4) printf(" %12.4f",in_value);       // info is stored in start.fp
			if (i==4) printf("\n");} }


	for (j = 1; j <= n22; j++)  {// j labels the footprint
		for (i = 1; i <= npart2 ; i++)  // i labels the particle
            {
 		fread(&in_value,sizeof(float),1,footinfile2);
            data2[i][j] = in_value;                 // notice data[i][j] is transpose of how 
		} }
         
/* What is the standard deviation of footprint? Fix footprint variable, j, vary
          particle variable i.
*/
if (0) {
	printf("new footprint of particles 1, 2,3,4 is \n");
 	for (j = 1; j <= m; j++)  {// j labels the footprint
		stddev = 0.;
		for (i = 1; i <= n; i++)  // i labels the particle
			{ stddev = data[i][j]*data[i][j] ;
//		for (i = 1; i <= n; i++)  // i labels the particle
			 data[i][j]*=  sqrt(sqrt(1/stddev));			
			if (i<3) printf(" %f ",data[i][j]);       // info is stored in start.fp
			if (i==2) printf("\n"); }
	}
	}	
		   /* Check on (part of) input data.
    for (i = 1; i <= 18; i++) {for (j = 1; j <= 8; j++)  {
        printf("%7.1f", data[i][j]);  }  printf("\n");  }
    */

    symmat = matrix(m, m);  /* Allocation of correlation (etc.) matrix */

   /* Look at analysis option; branch in accordance with this. */
   option = 'S';
  /* Look at analysis option; branch in accordance with this. */

     switch(option)
             {
          case 'R':
          case 'r':
              printf("Analysis of correlations chosen.\n");
              corcol(data, n, m, symmat);

                          /* Output correlation matrix.
                          for (i = 1; i <= m; i++) {
                           for (j = 1; j <= 8; j++)  {
                            printf("%7.4f", symmat[i][j]);  }
                            printf("\n");  }
                          */
              break;
          case 'V':
          case 'v':
              printf("Analysis of variances-covariances chosen.\n");
              covcol(data, n, m, symmat);

                          /* Output variance-covariance matrix.
                          for (i = 1; i <= m; i++) {
                          for (j = 1; j <= 8; j++)  {
                            printf("%7.1f", symmat[i][j]);  }
                            printf("\n");  }
                          */
              break;
          case 'S':
          case 's':
              printf("Analysis of sums-of-squares-cross-products");
              printf(" matrix chosen.\n");
              scpcol(data, n, m, symmat);

                         /* Output SSCP matrix.
                         for (i = 1; i <= m; i++) {
                          for (j = 1; j <= 8; j++)  {
                            printf("%7.1f", symmat[i][j]);  }
                            printf("\n");  }
                         */
              break;
          default:
/*              printf("Option: %s\n",option); */
              printf("For option, please type R, V, or S\n");
              printf("(upper or lower case).\n");
              printf("Exiting to system.\n");
              exit(1);
              break;
          }

/*********************************************************************
    Eigen-reduction
**********************************************************************/

    /* Allocate storage for dummy and new vectors. */
    evals = vector(m);     /* Storage alloc. for vector of eigenvalues */
    interm = vector(m);    /* Storage alloc. for 'intermediate' vector */
    symmat2 = matrix(m, m);  /* Duplicate of correlation (etc.) matrix */
    for (i = 1; i <= m; i++) {
     for (j = 1; j <= m; j++) {
      symmat2[i][j] = symmat[i][j]; /* Needed below for col. projections */
                              }}
							  							  
    tred2(symmat, m, evals, interm);  /* Triangular decomposition */
	
//   for (i = 1; i <= m; i++) { for (j = 1; j <= m; j++) {
//      printf("symmat[%d][%d] = %f\n",i,j,symmat[i][j]); /* Needed below for col. projections */
//                              } }
							  
    tqli(evals, interm, m, symmat);   /* Reduction of sym. trid. matrix */
    /* evals now contains the eigenvalues,
       columns of symmat now contain the associated eigenvectors. */

//       We need to reorder the eigenvalues and symmat
// but not  data
	for (i =1; i <= m ; i++){ // switching labels
       evalmax = -100;
		for (j =i; j <= m ; j++){ //switching labels 
			if (evals[j] > evalmax) {
				evalmax = evals[j];
				jmax = j;
			}
		}	
// now switch i and jmax entries
		evals[jmax] = evals[i];
		evals[i]    = evalmax;
// now switch i and jmax rows of symmat
		for (index=1;index <= m;index ++){
			column[index]    = symmat[index][i];
			symmat[index][i] = symmat[index][jmax];
			symmat[index][jmax] = column[index];
		}
	}

     printf("\nEigenvalues:\n"); fflush(stdout); 
     for (j = 1; j <= m; j++) {
       printf("%18.5f\n", evals[j]); }
//     printf("\n(Eigenvalues should be strictly positive; limited\n");
//     printf("precision machine arithmetic may affect this.\n");
//     printf("Eigenvalues are often expressed as cumulative\n");
//     printf("percentages, representing the 'percentage variance\n");
//     printf("explained' by the associated axis or principal component.)\n");


     printf("\nEigenvectors:\n");
     printf("(First %d; their definition in terms of original vbes.)\n", CUTOFF);
//            The eigenvectors are written as columns
	for (j=1; j <=6 && j <= m; j++)  printf("Eigen %d:\t", j);  
	printf("\n");
     for (j = 1; j <= m; j++) {
       for (i = 1; i <= CUTOFF && i <= 8; i++)  {
          printf("%12.4f", symmat[j][i]);  } // modified by prb
          printf("\n");  }

     /* Form projections of row-points on first three prin. components. */
     /* Store in 'data', overwriting original data. */
     for (i = 1; i <= n; i++) {
      for (j = 1; j <= m; j++) {
        interm[j] = data[i][j]; }   /* data[i][j] will be overwritten */
        for (k = m-CUTOFF+1; k <= m; k++) { // 
          data[i][k] = 0.0;
          for (k2 = 1; k2 <= m; k2++) {
            data[i][k] += interm[k2] * symmat[k2][m-k+1]; }
        }
     }


     for (i = 1; i <= npart2; i++) {
      for (j = 1; j <= n22; j++) {
        interm[j] = data2[i][j]; }   /* data[i][j] will be overwritten */
        for (k = n22-CUTOFF+1; k <= n22; k++) { // 
          data2[i][k] = 0.0;
          for (k2 = 1; k2 <= n22; k2++) {
            data2[i][k] += interm[k2] * symmat[k2][n22-k+1]; }
        }
     }

     printf("\n Projections of row-points on first %d prin. comps.:\n",CUTOFF);
//           The rows across are the decomposition of the rows in terms of the eigenvectors;

	for (i=1; i <=6 && i <= n; i++)  printf("Particle %d:\t", i);  
	printf("\n");
	
	for (j = 1; j <= CUTOFF; j++)  {   // change m back to CUTOFF later
	     for (i = 1; i <= 6 && i <= n; i++) {      // remember i means particle
          printf("%12.4f    ", data[i][m+1-j] );  }  // modified by PRB
          printf("\n") ; fflush(stdout);  
		  }

		  
	printf("\n n is %d, n2 is %d , CUTOFF is %d \n", n , m , CUTOFF);

	for (i=1; i <=8 && i <= n; i++)  printf("Part  %d:\t", i);  
	printf("\n");
	
	for (j = 1; j <= CUTOFF; j++) {
		for (i = 1; i <= n; i++)  {
//			if (j<=CUTOFF) printf("i is %d, j is %d, q is %f \n",i,j, q);
//			fflush(stdout);
			q= data[i][m+1-j] ;
			fwrite(&q,sizeof(float),1,footoutfile);
			if (i<=8) printf("%12.4f ", q);
		}         printf("\n");  }

	for (j = 1; j <= CUTOFF; j++) {	for (i = 1; i <= npart2; i++)  {
			q= data2[i][n22+1-j] ;
			fwrite(&q,sizeof(float),1,footoutfile2); 
//			if (i<=8) printf("%12.4f ", q); 
			} 
//			printf("\n");
			}
/*     Form projections of col.-points on first three prin. components. */
/*     Store in 'symmat2', overwriting what was stored in this.         */
//	for (j = 1; j <= m; j++) {
//		for (k = 1; k <= m; k++) {
//			interm[k] = symmat2[j][k]; }  /*symmat2[j][k] will be overwritten*/
//		for (i = 1; i <= CUTOFF; i++) {
//			symmat2[j][i] = 0.0;
//			for (k2 = 1; k2 <= m; k2++) {
//				symmat2[j][i] += interm[k2] * symmat[k2][m-i+1]; }
//			if (evals[m-i+1] > 0.0005)   /* Guard against zero eigenvalue */
//				symmat2[j][i] /= sqrt(evals[m-i+1]);   /* Rescale */
 //         	else
//				symmat2[j][i] = 0.0;    /* Standard kludge */
//		}
//		}
//  This seems to be the square of the symmat matrix

//    printf("\nProjections of column-points on first %d prin. comps.:\n", CUTOFF);
//     for (j = 1; j <= m; j++) {
//       for (k = 1; k <= CUTOFF; k++)  {
//         printf("%12.4f", symmat2[j][m+1-k]);  } // Modified by PRB
//          printf("\n");  }

    free_matrix(data, n, m);
    free_matrix(symmat, m, m);
    free_matrix(symmat2, m, m);
    free_vector(evals, m);
    free_vector(interm, m);

LOG(Ref,NULL,LOG_END,NULL);

}

/**  Correlation matrix: creation  ***********************************/

void corcol(float **data,int n,int m,float **symmat)
/* Create m * m correlation matrix from given n * m data matrix. */
{
float eps = 0.005;
float x, *mean, *stddev;
int i, j, j1, j2;

/* Allocate storage for mean and std. dev. vectors */

mean = vector(m);
stddev = vector(m);

/* Determine mean of column vectors of input data matrix */

for (j = 1; j <= m; j++)
    {
    mean[j] = 0.0;
    for (i = 1; i <= n; i++)
        {
        mean[j] += data[i][j];
        }
    mean[j] /= (float)n;
    }

printf("\nMeans of column vectors:\n");
for (j = 1; j <= m; j++)  {
    printf("%7.1f",mean[j]);  }   printf("\n");

/* Determine standard deviations of column vectors of data matrix. */

for (j = 1; j <= m; j++)
    {
    stddev[j] = 0.0;
    for (i = 1; i <= n; i++)
        {
        stddev[j] += (   ( data[i][j] - mean[j] ) *
                         ( data[i][j] - mean[j] )  );
        }
        stddev[j] /= (float)n;
        stddev[j] = sqrt(stddev[j]);
        /* The following in an inelegant but usual way to handle
        near-zero std. dev. values, which below would cause a zero-
        divide. */
        if (stddev[j] <= eps) stddev[j] = 1.0;
    }

printf("\nStandard deviations of columns:\n");
for (j = 1; j <= m; j++) { printf("%7.1f", stddev[j]); }
printf("\n");

/* Center and reduce the column vectors. */

for (i = 1; i <= n; i++)
    {
    for (j = 1; j <= m; j++)
        {
        data[i][j] -= mean[j];
        x = sqrt((float)n);
        x *= stddev[j];
        data[i][j] /= x;
        }
    }

/* Calculate the m * m correlation matrix. */
for (j1 = 1; j1 <= m-1; j1++)
    {
    symmat[j1][j1] = 1.0;
    for (j2 = j1+1; j2 <= m; j2++)
        {
        symmat[j1][j2] = 0.0;
        for (i = 1; i <= n; i++)
            {
            symmat[j1][j2] += ( data[i][j1] * data[i][j2]);
            }
        symmat[j2][j1] = symmat[j1][j2];
        }
    }
    symmat[m][m] = 1.0;

return;

}

/**  Variance-covariance matrix: creation  *****************************/

void covcol(float **data,int n,int m,float **symmat)
/* Create m * m covariance matrix from given n * m data matrix. */
{
float *mean;
int i, j, j1, j2;

/* Allocate storage for mean vector */

mean = vector(m);

/* Determine mean of column vectors of input data matrix */

for (j = 1; j <= m; j++)
    {
    mean[j] = 0.0;
    for (i = 1; i <= n; i++)
        {
        mean[j] += data[i][j];
        }
    mean[j] /= (float)n;
    }

printf("\nMeans of column vectors:\n");
for (j = 1; j <= m; j++)  {
    printf("%7.1f",mean[j]);  }   printf("\n");

/* Center the column vectors. */

for (i = 1; i <= n; i++)
    {
    for (j = 1; j <= m; j++)
        {
        data[i][j] -= mean[j];
        }
    }

/* Calculate the m * m covariance matrix. */
for (j1 = 1; j1 <= m; j1++)
    {
    for (j2 = j1; j2 <= m; j2++)
        {
        symmat[j1][j2] = 0.0;
        for (i = 1; i <= n; i++)
            {
            symmat[j1][j2] += data[i][j1] * data[i][j2];
            }
        symmat[j2][j1] = symmat[j1][j2];
        }
    }

return;

}

/**  Sums-of-squares-and-cross-products matrix: creation  **************/

void scpcol(float **data,int n,int m,float **symmat)
/* Create m * m sums-of-cross-products matrix from n * m data matrix. */
{
int i, j1, j2;

/* Calculate the m * m sums-of-squares-and-cross-products matrix. */

for (j1 = 1; j1 <= m; j1++)
    {
    for (j2 = j1; j2 <= m; j2++)
        {
        symmat[j1][j2] = 0.0;
        for (i = 1; i <= n; i++)
            {
            symmat[j1][j2] += data[i][j1] * data[i][j2];
            }
        symmat[j2][j1] = symmat[j1][j2];
        }
    }

return;

}

/**  Error handler  **************************************************/

void erhand(char err_msg[])
/* Error handler */
{
    fprintf(stderr,"Run-time error:\n");
    fprintf(stderr,"%s\n", err_msg);
    fprintf(stderr,"Exiting to system.\n");
    exit(1);
}

/**  Allocation of vector storage  ***********************************/

float *vector(int n)
/* Allocates a float vector with range [1..n]. */
{

    float *v;

    v = (float *) malloc ((unsigned) n*sizeof(float));
    if (!v) erhand("Allocation failure in vector().");
    return v-1;

}

/**  Allocation of float matrix storage  *****************************/

float **matrix(int n,int m)
/* Allocate a float matrix with range [1..n][1..m]. */
{
    int i;
    float **mat;

    /* Allocate pointers to rows. */
    mat = (float **) malloc((unsigned) (n)*sizeof(float*));
    if (!mat) erhand("Allocation failure 1 in matrix().");
    mat -= 1;

    /* Allocate rows and set pointers to them. */
    for (i = 1; i <= n; i++)
        {
        mat[i] = (float *) malloc((unsigned) (m)*sizeof(float));
        if (!mat[i]) erhand("Allocation failure 2 in matrix().");
        mat[i] -= 1;
        }

     /* Return pointer to array of pointers to rows. */
     return mat;

}

/**  Deallocate vector storage  *********************************/

void free_vector(float *v,int n)
/* Free a float vector allocated by vector(). */
{
   free((char*) (v+1));
}

/**  Deallocate float matrix storage  ***************************/

void free_matrix(float **mat,int n,int m)
/* Free a float matrix allocated by matrix(). */
{
   int i;

   for (i = n; i >= 1; i--)
       {
       free ((char*) (mat[i]+1));
       }
   free ((char*) (mat+1));
}

/**  Reduce a real, symmetric matrix to a symmetric, tridiag. matrix. */

void tred2(float **a,int n,float *d,float *e)
/* Householder reduction of matrix a to tridiagonal form.
   Algorithm: Martin et al., Num. Math. 11, 181-195, 1968.
   Ref: Smith et al., Matrix Eigensystem Routines -- EISPACK Guide
        Springer-Verlag, 1976, pp. 489-494.
        W H Press et al., Numerical Recipes in C, Cambridge U P,
        1988, pp. 373-374.  */
{
int l, k, j, i;
float scale, hh, h, g, f;

for (i = n; i >= 2; i--)
    {
    l = i - 1;
    h = scale = 0.0;
    if (l > 1)
       {
       for (k = 1; k <= l; k++)
           scale += fabs(a[i][k]);
       if (scale == 0.0)
          e[i] = a[i][l];
       else
          {
          for (k = 1; k <= l; k++)
              {
              a[i][k] /= scale;
              h += a[i][k] * a[i][k];
              }
          f = a[i][l];
          g = f>0 ? -sqrt(h) : sqrt(h);
          e[i] = scale * g;
          h -= f * g;
          a[i][l] = f - g;
          f = 0.0;
          for (j = 1; j <= l; j++)
              {
              a[j][i] = a[i][j]/h;
              g = 0.0;
              for (k = 1; k <= j; k++)
                  g += a[j][k] * a[i][k];
              for (k = j+1; k <= l; k++)
                  g += a[k][j] * a[i][k];
              e[j] = g / h;
              f += e[j] * a[i][j];
              }
          hh = f / (h + h);
          for (j = 1; j <= l; j++)
              {
              f = a[i][j];
              e[j] = g = e[j] - hh * f;
              for (k = 1; k <= j; k++)
                  a[j][k] -= (f * e[k] + g * a[i][k]);
              }
         }
    }
    else
        e[i] = a[i][l];
    d[i] = h;
    }
d[1] = 0.0;
e[1] = 0.0;
for (i = 1; i <= n; i++)
    {
    l = i - 1;
    if (d[i])
       {
       for (j = 1; j <= l; j++)
           {
           g = 0.0;
           for (k = 1; k <= l; k++)
               g += a[i][k] * a[k][j];
           for (k = 1; k <= l; k++)
               a[k][j] -= g * a[k][i];
           }
       }
       d[i] = a[i][i];
       a[i][i] = 1.0;
       for (j = 1; j <= l; j++)
           a[j][i] = a[i][j] = 0.0;
    }
}

/**  Tridiagonal QL algorithm -- Implicit  **********************/

void tqli(float d[],float e[],int n,float **z)
{
int m, l, iter, i, k;
float s, r, p, g, f, dd, c, b;

for (i = 2; i <= n; i++)
    e[i-1] = e[i];
e[n] = 0.0;
for (l = 1; l <= n; l++)
    {
    iter = 0;
    do
      {
      for (m = l; m <= n-1; m++)
          {
          dd = fabs(d[m]) + fabs(d[m+1]);
//          if (fabs(e[m]) + dd == dd) break;
          if (fabs(e[m]) /dd <= 1.e-6) break;
          }
          if (m != l)
             {
             if (iter++ == CONVERGE) erhand("No convergence in TLQI.");
             g = (d[l+1] - d[l]) / (2.0 * e[l]);
             r = sqrt((g * g) + 1.0);
             g = d[m] - d[l] + e[l] / (g + SIGN(r, g));
             s = c = 1.0;
             p = 0.0;
             for (i = m-1; i >= l; i--)
                 {
                 f = s * e[i];
                 b = c * e[i];
                 if (fabs(f) >= fabs(g))
                    {
                    c = g / f;
                    r = sqrt((c * c) + 1.0);
                    e[i+1] = f * r;
                    c *= (s = 1.0/r);
                    }
                 else
                    {
                    s = f / g;
                    r = sqrt((s * s) + 1.0);
                    e[i+1] = g * r;
                    s *= (c = 1.0/r);
                    }
                 g = d[i+1] - p;
                 r = (d[i] - g) * s + 2.0 * c * b;
                 p = s * r;
                 d[i+1] = g + p;
                 g = c * r - b;
                 for (k = 1; k <= n; k++)
                     {
                     f = z[k][i+1];
                     z[k][i+1] = s * z[k][i] + c * f;
                     z[k][i] = c * z[k][i] - s * f;
                     }
                 }
                 d[l] = d[l] - p;
                 e[l] = g;
                 e[m] = 0.0;
             }
          }  while (m != l);
      }
 }

/*********************************/
/* Principal Components Analysis */
/*********************************/

/*********************************************************************/
/* Principal Components Analysis or the Karhunen-Loeve expansion is a
   classical method for dimensionality reduction or exploratory data
   analysis.  One reference among many is: F. Murtagh and A. Heck,
   Multivariate Data Analysis, Kluwer Academic, Dordrecht, 1987 
   (hardbound, paperback and accompanying diskette).

   This program is public-domain.  If of importance, please reference 
   the author.  Please also send comments of any kind to the author:

   F. Murtagh
   Schlossgartenweg 1          or        35 St. Helen's Road
   D-8045 Ismaning                       Booterstown, Co. Dublin
   W. Germany                            Ireland

   Phone:        + 49 89 32006298 (work)
                 + 49 89 965307 (home)
   Telex:        528 282 22 eo d
   Fax:          + 49 89 3202362
   Earn/Bitnet:  fionn@dgaeso51,  fim@dgaipp1s,  murtagh@stsci
   Span:         esomc1::fionn
   Internet:     murtagh@scivax.stsci.edu
   

   A Fortran version of this program is also available.     
    
   F. Murtagh, Munich, 6 June 1989                                   */   
/*********************************************************************/
//      printf("(filename -- give full path name,\n");
//      printf(" #rows                          \n");
//      printf(" #cols    -- integer values,\n");                  
//      printf(" option   -- R (recommended) for correlation analysis,\n");
//      printf("             V for variance/covariance analysis\n");
//      printf("             S for SSCP analysis.)\n");
//   n = atoi(argv[2]);              /* # rows  in pca.c*/  
//   m = atoi(argv[3]);              /* # columns  in pca.c */
//   strncpy(&option,argv[4],1);     /* Analysis option */


/* This is a different qsort algorithm

struct A { float v, int i};

int cmp(A *a1,A *a2) { return (a1->v < a2->v)?-1:((a1->v > a2->v)?1:0)  ; }

main() {
A *a;
a = malloc(sizeof(A) * nelements);
// copy data into +a;No match

for (j=0; j<nelements; j++) { a[j].v=value[j]; a[j].i=j; }

qsort(a,nelements,sizeof(a),cmp);

}
*/
